@@ -356,7 +356,9 @@ def flexibleParamTree(treeParams: t.Union[t.List[Parameter], Parameter]=None, sh
       if 'added' in desc.lower():
         setParamTooltips(tree)
         if resizeNameCol:
-          tree.resizeColumnToContents(0)
+          # Bug: Current 'resize to contents' makes the name column just a bit small
+          hint = tree.sizeHintForColumn(0)
+          tree.setColumnWidth(0, int(hint*1.1))
     param.sigTreeStateChanged.connect(maybeUpdateTips)
 
   if setTooltips:
