@@ -7,6 +7,7 @@ from pathlib import Path
 from pyqtgraph.Qt import QtCore, QtWidgets
 from pyqtgraph.parametertree import Parameter
 from pyqtgraph.parametertree.parameterTypes import GroupParameter
+import pyqtgraph as pg
 
 from utilitys.widgets import EasyWidget, DummySignal
 from . import PrjParam
@@ -56,7 +57,8 @@ param editor.
 
 class ParamEditor(QtWidgets.QDockWidget):
   sigParamStateCreated = Signal(str)
-  sigChangesApplied = Signal(dict)
+  sigChangesApplied = Signal(object)
+  # Dict of changes, or *None*
   sigParamStateDeleted = Signal(str)
 
   _baseRegisterPath: t.Sequence[str] = ()
@@ -132,14 +134,19 @@ class ParamEditor(QtWidgets.QDockWidget):
       self.saveDir.mkdir(parents=True, exist_ok=True)
       self.watcher.addPath(str(self.saveDir))
     self.watcher.directoryChanged.connect(lambda: self.updateSavedStates())
+    def maybeUpdateCurState(file: str):
+      if Path(file) == self.formatFileName(self.stateName):
+        # Time for a reload, Put in event loop to allow file time to save
+        # TODO: 5 ms might not be long enough for a file save, figure out a good
+        #   way of waiting the right amount of time
+        QtCore.QTimer.singleShot(5, lambda: self.loadParamValues(self.stateName))
+    self.watcher.fileChanged.connect(maybeUpdateCurState)
 
     # -----------
     # Construct parameter tree
     # -----------
     self.params = Parameter.create(name='Parameters', type='group', children=paramList)
 
-    self.params.sigStateChanged.connect(self._paramTreeChanged)
-
     if treeChildren is None:
       treeChildren = self.params
     if isinstance(treeChildren, Parameter):
@@ -149,8 +156,8 @@ class ParamEditor(QtWidgets.QDockWidget):
         treeChildren[ii] = self.params
     self.tree = fns.flexibleParamTree(treeChildren, showTop=False)
 
-    self._stateBeforeEdit = self.params.saveState()
-    self.lastAppliedName = None
+    self.stateName = None
+    self._lastAppliedState: t.Optional[dict]=None
 
     self._buildGui()
 
@@ -161,11 +168,13 @@ class ParamEditor(QtWidgets.QDockWidget):
         pType: popuplineeditor
         limits: [Default]
       """
-      if loadState in self.savedStates and loadState != self.lastAppliedName:
+      if loadState in self.savedStates and loadState != self.stateName:
         return self.loadParamValues(loadState)
 
     proc, param = self.registerFunc(niceStateLoader, namePath=(), overrideBasePath=(), nest=False, returnParam=True,
                                  parentParam=self._metaParamGrp, runOpts=RunOpts.ON_CHANGED)
+    # Hide this from the global registry to avoid menu option creation
+    del self.procToParamsMapping[proc]  
     self.stateSelector = param.child('loadState')
 
     # Populate initial values, avoid early trigger of load values
@@ -177,21 +186,39 @@ class ParamEditor(QtWidgets.QDockWidget):
 
     SPAWNED_EDITORS.append(weakref.proxy(self))
 
-  def _paramTreeChanged(self, *_args, **_kwargs):
-    self._stateBeforeEdit = self.params.saveState()
+  def applyChanges(self, newName: FilePath=None, newState: dict=None, recreate=False):
+    """
+    Broadcasts that this parameter editor has updated changes
 
-  def applyChanges(self):
-    """Broadcasts that this parameter editor has updated changes"""
-    # Don't emit any signals if nothing changed
-    newState = self.params.saveState(filter='user')
-    outDict = self.params.getValues()
-    if self._stateBeforeEdit != newState:
-      self._stateBeforeEdit = newState
-      self.sigChangesApplied.emit(outDict)
-    return outDict
+    :param newName: Name of the applied state. If *None*, stateName is used
+    :param newState: Dict representation of the new state. If *None*, the most recently
+      applied state is used
+    """
+    if newName is None:
+      newName = self.stateName
+    if newState is None:
+      newState = self._lastAppliedState
+
+    if newName:
+      newName = Path(newName).stem
+
+    self.sigChangesApplied.emit(newState)
+
+    if self.stateName == newName:
+      return
+    oldName = self.formatFileName(self.stateName)
+    # Watch the new name, take out the old name. OK if neither exists, will just return
+    # false
+    self.watcher.removePath(str(oldName))
+    self.watcher.addPath(str(self.formatFileName(newName)))
+
+    self.stateName = newName
+    self.stateSelector.setValue(self.stateName)
+    return newState
 
   def saveParamValues(self, saveName: str=None, paramState: dict=None, *, allowOverwriteDefault=False,
-                      includeDefaults=False, blockWrite=False) -> t.Optional[t.Dict[str, t.Any]]:
+                      includeDefaults=False, blockWrite=False, applyChanges=True
+                      ) -> t.Optional[t.Dict[str, t.Any]]:
     """
     * Returns dict on successful parameter save and emits sigParamStateCreated.
     * Returns None if no save name was given
@@ -200,13 +227,16 @@ class ParamEditor(QtWidgets.QDockWidget):
       return None
     elif paramState is None:
       paramState = fns.paramValues(self.params, includeDefaults=includeDefaults or allowOverwriteDefault)
+      # Nothing changed for this state, no need to do any updating
+      applyChanges = False
     # Remove non-useful values
     if not blockWrite and self.saveDir is not None:
       fns.saveToFile(paramState, self.formatFileName(saveName),
                  allowOverwriteDefault=allowOverwriteDefault)
-    # self.applyChanges()
-    self.lastAppliedName = saveName
-    # TODO: Decide whether states created outside saveDir should be emitted here
+    if applyChanges:
+      # Since save state shouldn't be modifying the current state, it will emit the same
+      # state as before, but under a new name -- regardless of the param state dict
+      self.applyChanges(saveName)
     return paramState
 
   def saveCurStateAsDefault(self):
@@ -219,8 +249,7 @@ class ParamEditor(QtWidgets.QDockWidget):
     with self.params.treeChangeBlocker():
       fns.applyParamOpts(self.params, loadDict)
     if applyChanges:
-      self.applyChanges()
-    self.lastAppliedName = stateName
+      self.applyChanges(stateName, stateDict)
     return loadDict
 
   def _parse_updateCandidateParams(self, loadDict: dict, candidateParams: t.Sequence[Parameter]):
@@ -256,7 +285,7 @@ class ParamEditor(QtWidgets.QDockWidget):
                       stateDict: dict=None,
                       useDefaults=True,
                       candidateParams: t.List[Parameter]=None,
-                      applyChanges=True, ):
+                      applyChanges=True):
     """
     Can restore a state created by `fns.paramValues`
 
@@ -287,17 +316,13 @@ class ParamEditor(QtWidgets.QDockWidget):
       self._parse_updateCandidateParams(loadDict, candidateParams)
 
     if applyChanges:
-      self.applyChanges()
-      if stateName:
-        stateName = Path(stateName).stem
-      self.lastAppliedName = stateName
-      self.stateSelector.setValue(self.lastAppliedName)
+      self.applyChanges(stateName, loadDict)
     return fns.paramValues(self.params, includeDefaults=True)
 
 
   def formatFileName(self, stateName: t.Union[str, Path]=None):
     if stateName is None:
-      stateName = self.lastAppliedName
+      stateName = self.stateName
     suffix = f'.{self.fileType}'
     stateName = stateName or suffix
     stateName = Path(stateName)
@@ -626,13 +651,6 @@ class ParamEditor(QtWidgets.QDockWidget):
     Tree for controlling meta-parameters, useful for configuring the parameter editor itself
     (i.e. loading state, etc.)
     """
-    self._metaTree.setHeaderHidden(True)
-    oldSzHint = self._metaTree.sizeHint
-    def newSzHint():
-      baseHint = oldSzHint()
-      baseHint.setHeight(int(baseHint.height() * 1.1))
-      return baseHint
-    self._metaTree.sizeHint = newSzHint
     # self._metaTree.setSizePolicy(self._metaTree.sizePolicy().horizontalPolicy(), QtWidgets.QSizePolicy.Fixed)
     return self._metaTree, self._metaParamGrp
 
@@ -667,7 +685,7 @@ class ParamEditor(QtWidgets.QDockWidget):
     super().reject()
 
   def saveParamValues_gui(self):
-    saveName = fns.dialogGetSaveFileName(self, 'Save As', self.lastAppliedName)
+    saveName = fns.dialogGetSaveFileName(self, 'Save As', self.stateName)
     self.saveParamValues(saveName)
 
 class ParamEditorDockGrouping(QtWidgets.QDockWidget):
