@@ -103,6 +103,7 @@ class ProcessStage(ABC):
   input: ProcessIO = None
   allowDisable = False
   disabled = False
+  cacheOnDisable = False
   result: ProcessIO = None
   mainResultKeys: StrList = None
   mainInputKeys: StrList = None
@@ -278,12 +279,18 @@ class AtomicProcess(ProcessStage):
   def keysFromPrevIO(self):
     return self.input.keysFromPrevIO
 
-  def run(self, prevIO: ProcessIO=None, disable=False, **runKwargs):
+  def run(self, io: ProcessIO=None, disable=False, **runKwargs):
     ProcessIO.count = self
     disable = disable or self.disabled
-    self.updateInput(prevIO, **runKwargs)
-    self.result = self.func(**self.input) if not disable else self.input
-    return self.result
+    self.updateInput(io, **runKwargs)
+    if not disable:
+      self.result = self.func(**self.input)
+    elif not self.cacheOnDisable:
+      self.result = None
+    out = self.result
+    if out is None:
+      out = self.input
+    return out
 
   def saveState(self, includeDefaults=False, includeMeta=False, **metaFilterOpts):
     def keepCond(k, v):
@@ -368,21 +375,37 @@ class NestedProcess(ProcessStage):
     self.stages[0].updateInput(prevIo, graceful=graceful)
 
   def run(self, io: ProcessIO = None, disable=False, **runKwargs):
-    _activeIO = ProcessIO() if not io else _copy.copy(io)
-    _activeIO.update(runKwargs)
+    _activeIo = ProcessIO() if not io else _copy.copy(io)
+    _activeIo.update(runKwargs)
 
     if self.disabled or disable and not self.result:
       # TODO: Formalize this behavior. Disabling before being run should avoid eating the input args
-      return _activeIO
+      return _activeIo
 
     for i, stage in enumerate(self):
-      newIO = stage.run(_activeIO, disable=disable or self.disabled)
-      if isinstance(newIO, ProcessIO):
-        _activeIO.update(newIO)
+      newIo = stage.run(_activeIo, disable=disable or self.disabled)
+      if not isinstance(newIo, ProcessIO) and isinstance(stage, AtomicProcess):
+        # Try wrapping the inner function for future runs
+        newIo = self._maybeWrapStageFunc(stage, newIo)
+      if isinstance(newIo, ProcessIO):
+        _activeIo.update(newIo)
 
-    self.result = _activeIO
+    self.result = _activeIo
     return self.result
 
+  def _maybeWrapStageFunc(self, stage: AtomicProcess, oldresult: t.Any):
+    oldFunc = stage.func
+    useKeys = stage.mainResultKeys or self.mainResultKeys
+    try:
+      stage.func = stage._wrappedFunc(oldFunc, useKeys)
+      # Capture current result as a process io
+      def dummyFunc():
+        return oldresult
+      oldresult = stage._wrappedFunc(dummyFunc, useKeys)()
+    except Exception:
+      stage.func = oldFunc
+    return oldresult
+
   def saveState(self, includeDefaults=False, includeMeta=False, **metaFilterOpts):
     stageStates = []
     for stage in self:
